/*
 * show.c
 *
 * Copyright 2000, 2003 by Anthony Howe.  All rights reserved.
 *
 * usage: show lines files...
 */

#include <com/snert/lib/version.h>

#include <fcntl.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/stat.h>
#include <sys/types.h>

#include <com/snert/lib/io/posix.h>

char buf[BUFSIZ];

char usage[] =
"\033[1musage: show count files...\033[0m\n"
"\n"
"count\t\tDisplay the top (+ve) or bottom (-ve) N lines; 0 for all\n"
"files ...\tlist of files to show\n"
"\n"
"\033[1mshow/1.1 Copyright 2000, 2003 by Anthony Howe. All rights reserved.\033[0m\n"
;

int
main(int argc, char **argv)
{
	size_t n;
	int i, fd;
	off_t offset;
	long lines, count;
	struct stat finfo;
	char *b, *eb;

	if (argc < 3) {
		fprintf(stderr, usage);
		return 2;
	}

	lines = strtol(argv[1], (char **) 0, 10);

	for (i = 2; i < argc; ++i) {
		if (stat(argv[i], &finfo) < 0)
			return 1;

		if ((fd = open(argv[i], O_RDONLY)) < 0)
			return 1;

		if (lines == 0) {
			while (0 < (n = read(fd, buf, BUFSIZ))) {
				(void) write(1, buf, n);
			}
		} else {
			if (lines < 0) {
				count = lines = -lines;

				/* Start with the odd buffer length from the end of file. */
				offset = finfo.st_size % BUFSIZ;

				if ((offset = lseek(fd, -offset, SEEK_END)) == (off_t) -1)
					return 1;

				for (;;) {
					/* Fill the buffer. */
					if ((n = read(fd, buf, BUFSIZ)) == (size_t) -1)
						break;

					/* Count backwards newlines (fine for Unix, PC). */
					for (eb = buf+n; buf < eb; ) {
						if (*--eb == '\n' && --count < 0) {
							(void) lseek(fd, offset+(eb+1-buf), SEEK_SET);
							break;
						}
					}

					if (count <= 0)
						break;

					/* Move backwards by BUFSIZ byte units until we
					 * seek beyond the start of the file.
					 */
					if ((offset = lseek(fd, offset-BUFSIZ, SEEK_SET)) == (off_t) -1)
						break;
				}
			}

			while (0 < lines) {
				if ((n = read(fd, buf, BUFSIZ)) <= 0)
					break;

				/* Count newlines (fine for Unix, PC). */
				for (b = buf, eb = buf+n; b < eb; )
					if (*b++ == '\n' && --lines <= 0)
						break;

				(void) write(1, buf, b - buf);
			}
		}

		(void) close(fd);
	}

	return 0;
}
